//+------------------------------------------------------------------+
//|                                                    TMA_bands.mq5 |
//|                                                        avoitenko |
//|                        https://login.mql5.com/en/users/avoitenko |
//+------------------------------------------------------------------+
#property copyright "avoitenko"
#property link      "https://login.mql5.com/en/users/avoitenko"
#property version   "1.00"
#property indicator_chart_window

//---
#property indicator_buffers   8
#property indicator_plots     5
//---
#property indicator_type1 DRAW_LINE
#property indicator_type2 DRAW_LINE
#property indicator_type3 DRAW_LINE
#property indicator_type4 DRAW_LINE
#property indicator_type5 DRAW_LINE
//---
#property indicator_style1 STYLE_SOLID
#property indicator_style2 STYLE_DOT
#property indicator_style3 STYLE_SOLID
#property indicator_style4 STYLE_DOT
#property indicator_style5 STYLE_SOLID
//---
#property indicator_color1 clrRed
#property indicator_color2 clrRed
#property indicator_color3 clrDimGray
#property indicator_color4 clrBlue
#property indicator_color5 clrGreen
//---
#property indicator_label1 "Upper 2"
#property indicator_label2 "Upper 1"
#property indicator_label3 "Center"
#property indicator_label4 "Lower 1"
#property indicator_label5 "Lower 2"
//---
#define PREFIX "myPriceTag_"
//+------------------------------------------------------------------+
//|   Input parameters                                               |
//+------------------------------------------------------------------+
input uint                 InpHalfLength        =  56;            // Period
input ENUM_APPLIED_PRICE   InpPrice             =  PRICE_WEIGHTED;// Price
input double               InpBandsDeviations   =  2.5;           // Deviation
input bool                 InpShowPrice         =  true;          // Show price tags

//---
double tmBuffer[];
double upBuffer[];
double dnBuffer[];
double wuBuffer[];
double wdBuffer[];
double upLevel[];
double dnLevel[];
double prices[];

//--- global vars
int HalfLength;
int FullLength;
int ma_handle;
//+------------------------------------------------------------------+
//|   OnInit                                                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---   
   HalfLength = (int)fmax(InpHalfLength,1);
   FullLength = 2*HalfLength+1;
//---   
   SetIndexBuffer(0,upBuffer);
   SetIndexBuffer(1,upLevel);
   SetIndexBuffer(2,tmBuffer);
   SetIndexBuffer(3,dnLevel);
   SetIndexBuffer(4,dnBuffer);
//---
   SetIndexBuffer(5,wuBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,wdBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,prices,INDICATOR_CALCULATIONS);
//---
   ArraySetAsSeries(tmBuffer,true);
   ArraySetAsSeries(upBuffer,true);
   ArraySetAsSeries(dnBuffer,true);
   ArraySetAsSeries(upLevel,true);
   ArraySetAsSeries(dnLevel,true);
   ArraySetAsSeries(wuBuffer,true);
   ArraySetAsSeries(wdBuffer,true);
   ArraySetAsSeries(prices,true);
//---   
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,HalfLength);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,HalfLength);
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,HalfLength);
   PlotIndexSetInteger(3,PLOT_DRAW_BEGIN,HalfLength);
   PlotIndexSetInteger(4,PLOT_DRAW_BEGIN,HalfLength);
//---
   ma_handle=iMA(NULL,0,1,0,MODE_SMA,InpPrice);
   if(ma_handle==INVALID_HANDLE)return(INIT_FAILED);

   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//--- done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|   OnDeinit                                                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   int total= ObjectsTotal(0,0);
   for(int i=total-1; i>=0; i--)
     {
      string name=ObjectName(0,i);
      if(StringFind(name,PREFIX)==0)
         ObjectDelete(0,name);
     }
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//|   OnCalculate                                                    |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {

   ArraySetAsSeries(time,true);
//---   
   int limit,limit_price;
   if(prev_calculated<=0)
     {
      limit_price=rates_total-1;
      limit=rates_total-FullLength-1;
      ArrayInitialize(tmBuffer,EMPTY_VALUE);
      ArrayInitialize(upBuffer,EMPTY_VALUE);
      ArrayInitialize(dnBuffer,EMPTY_VALUE);
      ArrayInitialize(upLevel,EMPTY_VALUE);
      ArrayInitialize(dnLevel,EMPTY_VALUE);
      ArrayInitialize(wdBuffer,0);
      ArrayInitialize(wuBuffer,0);
     }
   else
     {
      limit=rates_total-prev_calculated+HalfLength;
      limit_price=rates_total-prev_calculated;
     }

//--- prices
   if(CopyBuffer(ma_handle,0,0,limit_price+1,prices)!=limit_price+1)return(0);

//--- main cycle
   for(int i=limit; i>=0 && !_StopFlag; i--)
     {
      double sum  = (HalfLength+1) * prices[i];
      double sumw = (HalfLength+1);

      //---
      for(int j=1,k=HalfLength; j<=HalfLength; j++,k--)
        {
         sum  += k*prices[i+j];
         sumw += k;

         if(i>=j)
           {
            sum  += k*prices[i-j];
            sumw += k;
           }
        }
      tmBuffer[i]=NormalizeDouble(sum/sumw,_Digits);

      double diff=prices[i]-tmBuffer[i];

      if(i==(rates_total-HalfLength-1))// first calc
        {
         upBuffer[i] = tmBuffer[i];
         dnBuffer[i] = tmBuffer[i];

         if(diff>=0)
           {
            wuBuffer[i] = pow(diff,2);
            wdBuffer[i] = 0;
           }
         else
           {
            wdBuffer[i] = pow(diff,2);
            wuBuffer[i] = 0;
           }
        }
      else
        {
         if(diff>=0)
           {
            wuBuffer[i] = (wuBuffer[i+1]*(FullLength-1) + pow(diff,2))/FullLength;
            wdBuffer[i] =  wdBuffer[i+1]*(FullLength-1)/FullLength;
           }
         else
           {
            wdBuffer[i] = (wdBuffer[i+1]*(FullLength-1)+pow(diff,2))/FullLength;
            wuBuffer[i] =  wuBuffer[i+1]*(FullLength-1)/FullLength;
           }
         upBuffer[i] = tmBuffer[i] + InpBandsDeviations*sqrt(wuBuffer[i]);
         dnBuffer[i] = tmBuffer[i] - InpBandsDeviations*sqrt(wdBuffer[i]);
        }

      double level=(upBuffer[i]-dnBuffer[i])*0.2; // other lines take away 20% OF DIFFERENCE IN RANGE
      upLevel[i] = (upBuffer[i] - level);
      dnLevel[i] = (dnBuffer[i] + level);
     }

//--- plot price tag
   if(InpShowPrice)
     {
      SetPrice("bandHigh",time[0],upBuffer[0],PlotIndexGetInteger(0,PLOT_LINE_COLOR));
      SetPrice("bandInnerHigh",time[0],upLevel[0],PlotIndexGetInteger(1,PLOT_LINE_COLOR));
      SetPrice("bandMid",time[0],tmBuffer[0],PlotIndexGetInteger(2,PLOT_LINE_COLOR));
      SetPrice("bandInnerLow",time[0],dnLevel[0],PlotIndexGetInteger(3,PLOT_LINE_COLOR));
      SetPrice("bandLow",time[0],dnBuffer[0],PlotIndexGetInteger(4,PLOT_LINE_COLOR));
     }

//--- done     
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|   SetPrice                                                       |
//+------------------------------------------------------------------+
void SetPrice(string name,datetime time,double price,color colour)
  {
   name=PREFIX+name;
   if(ObjectFind(0,name)==0 || ObjectCreate(0,name,OBJ_ARROW_RIGHT_PRICE,0,time,price))
     {
      ObjectSetInteger(0,name,OBJPROP_COLOR,colour);
      ObjectSetInteger(0,name,OBJPROP_WIDTH,2);
      ObjectSetInteger(0,name,OBJPROP_TIME,time);
      ObjectSetDouble(0,name,OBJPROP_PRICE,price);
     }
  }
//+------------------------------------------------------------------+
